local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110303173751') == 'number' and '20110303173751' or date('%Y%m%d%H%M%S')

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:NewAddon(MAJOR_VERSION, 'AceConsole-3.0' --[[,'AceEvent-3.0']] ,'AceHook-3.0')
--local L = LibStub('AceLocale-3.0'):GetLocale(MAJOR_VERSION)

local UnitXPMax = _G.UnitXPMax
local UnitLevel = _G.UnitLevel
local UnitGetGuildXP = _G.UnitGetGuildXP
local GetGuildLevel = _G.GetGuildLevel

--- Setup debug stuff.
do
    local libDebugLog = LibStub:GetLibrary('LibDebugLog-1.0',true)
    if libDebugLog then
        libDebugLog:Embed(addon)
    else
        function addon:Debug (...)
        end
    end
end

local chatShort = "glb"
local chatLong = MAJOR_VERSION
local addonDbName = MAJOR_VERSION .. 'DB'

addon.optionsTable = {
    name = MAJOR_VERSION,
    handler = addon,
    type = 'group',
    args = {
    },
}

addon.defaults = {
    profile = {
    },
}

--- Called for chat commands.
-- @param input the chat command
function addon:ChatCommand(input)
    if not input or input:trim() == '' then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub('AceConfigCmd-3.0').HandleCommand(MAJOR_VERSION, chatShort, MAJOR_VERSION, input)
    end
end

--- Called when the addon is loaded
-- (Ace3 does this)
function addon:OnInitialize()
    self.db = LibStub('AceDB-3.0'):New(addonDbName, addon.defaults)
    LibStub('AceConfig-3.0'):RegisterOptionsTable(MAJOR_VERSION, self.optionsTable)
    self.optionsFrame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(MAJOR_VERSION, MAJOR_VERSION)
    self:RegisterChatCommand(chatShort, 'ChatCommand')
    self:RegisterChatCommand(chatLong, 'ChatCommand')

    self:sanityCheck()
end

function addon:sanityCheck()
    -- sanitiy check
    local unitXPMax = UnitXPMax('player')
    if unitXPMax ~= 0 and unitXPMax ~= XP_PER_LEVEL[UnitLevel('player')] then
        error("The Player XP database is incorrect, please report this. Expected XP for level "
           .. UnitLevel('player') .. " is " .. XP_PER_LEVEL[UnitLevel('player')] .. " but UnitXPMax returned "
           .. unitXPMax)
    end

    local guildXP, guildXPRem = UnitGetGuildXP('player')
    local guildXPMax = guildXP + guildXPRem
    local guildLevel = GetGuildLevel('player')
    if guildXPMax and XP_PER_GUILD_LEVEL[guildLevel]
                  and guildXPMax ~= 0
                  and guildXPMax ~= XP_PER_GUILD_LEVEL[guildLevel] then
        error("The Guild XP database is incorrect, please report this. Expected XP for level "
           .. guildLevel .. " is " .. XP_PER_GUILD_LEVEL[guildLevel] .. " but game returned "
           .. guildXPMax)
    end
end


--- Called when the addon is enabled.
function addon:OnEnable()
end

--- Called when the addon is disabled
function addon:OnDisable()
end

function addon:merge(t1, t2)
    assert(not t1.super)
    assert(not t2.super)
    local result = {}
    for k,v in pairs(t1) do
        if not t2[k] then
            ChatFrame1:AddMessage("Copy key " .. k .. " right value " .. tostring(v))
            result[k] = self:deepCopy(v)
        end
    end
    for k,v in pairs(t2) do
        if t1[k] then
            if type(t1[k]) == 'table' then
                assert(type(v) == 'table')
                ChatFrame1:AddMessage("Merging table key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
                result[k] = self:merge(t1[k],v)
                ChatFrame1:AddMessage("Merged  table key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
            elseif type(t1[k]) == 'function' then
                assert(type(v) == 'function')
                ChatFrame1:AddMessage("Merging function key " .. k .. " values " .. tostring(t1[k]) .. " and " .. tostring(v))
                result[k] = v
                result.super = result.super or {}
                result.super[k] = t1[k]
            else
                ChatFrame1:AddMessage("Overwriting key " .. k .. " value " .. tostring(t1[k]) .. " with " .. tostring(v))
                result[k] = v
            end
        else
            ChatFrame1:AddMessage("Copy key " .. k .. " left value " .. tostring(v))
            result[k] = self:deepCopy(v)
        end
    end
    return result
end

function addon:deepCopy(arg)
    if type(arg) == 'table' then
        local result = {}
        for k,v in pairs(arg) do
            result[k] = self:deepCopy(v)
        end
        return result
    else
        return arg
    end
end
